package web

import (
	"fmt"
	"io"
	"log"
	"net/http"
	"strings"

	"github.com/gorilla/sessions"

	"code.osinet.fr/fgm/kurz/domain"
	"github.com/gorilla/mux"
)

// handlePostTarget handles form POST requests to /
func handlePostTarget(w http.ResponseWriter, r *http.Request, router *mux.Router) {
	var sess *sessions.Session

	sess, storeErr := store.Get(r, globals.SessionName)
	if storeErr != nil {
		log.Fatal()
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

	r.ParseForm()
	defer r.Body.Close()
	rawTarget := r.PostForm.Get(rootInputName)
	target, err := validateTarget(rawTarget)
	if err != nil {
		sess.AddFlash(err.Error())
		sess.Save(r, w)
		w.Header().Set("Location", URLFromRoute(router, RouteGetRoot, nil))
		w.WriteHeader(http.StatusSeeOther)
		return
	}

	short, isNew, err := domain.GetShortURL(target)
	if err != nil {
		w.WriteHeader(http.StatusInternalServerError)
		return
	}
	sw := &strings.Builder{}
	var templateName string
	if isNew {
		sess.AddFlash("New short URL created")
		templateName = "201"
	} else {
		sess.AddFlash("Existing short URL reused")
		templateName = "409"
	}
	defer sess.Save(r, w)

	data := struct {
		Flashes                 []interface{}
		FullyQualifiedShortURL  string
		FullyQualifiedTargetURL string
		Globals
	}{
		sess.Flashes(),
		short,
		target,
		globals,
	}

	err = tmpl.ExecuteTemplate(sw, templateName, data)
	if err != nil {
		fmt.Println(err)
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

	if isNew {
		w.WriteHeader(http.StatusCreated)
	} else {
		w.WriteHeader(http.StatusConflict)
	}
	io.Copy(w, strings.NewReader(sw.String()))
}

func validateTarget(raw string) (string, error) {
	if raw == "" {
		return "", domain.MakeError(domain.TargetInvalidError, "empty target")
	}
	// BUG(fgm): needs much more validation, starting with XSS.
	return raw, nil
}
